#
# module DB_csv - wrapper around comma separated value files
# with locking support. Data access will be done through
# hashes.
#
# Thomas Linden <tom@co.daemon.de>
#
# Copyright (c) 2003 Thomas Linden.
# ALL RIGHTS RESERVED.
#
# Licensed under the terms of the artistic license.
#


package DB_csv;

use FileHandle;
use Data::Dumper;
use Fcntl qw(:flock);
use strict;
use Carp;

$DB_csv::VERSION = "0.01";

sub new {
  my($this, %param ) = @_;

  foreach my $var (qw(File Lock Mode Struct Delimiter)) {
    if (!exists $param{$var}) {
      croak "Variable $var not defined!\n";
    }
  }

  # defaultize
  $param{FD} = new IO::File;
  if (ref($param{Struct}) ne "ARRAY") {
    croak "Struct must be an array reference!\n";
  }

  $param{Key}  = shift @{$param{Struct}};
  $param{Mask} = chr(182); # ""

  my $class = ref($this) || $this;
  my $self = \%param;
  bless($self,$class);

  $self->open();
  $self->lock();
  $self->read();

  return $self;
}

sub get {
  my ($this, $key) = @_;
  if (! $key) {
    croak "get() requires a key!\n";
  }
  if (exists $this->{data}->{$key}) {
    return %{$this->{data}->{$key}};
  }
  else {
    return ();
  }
}

sub set {
  my ($this, %param) = @_;
  if (! exists $param{ $this->{Key} }) {
    croak "Key argument required!\n";
  }
  foreach my $field (@{$this->{Struct}}) {
    if (! exists $param{$field}) {
      $param{$field} = "";
    }
  }
  my $key = delete $param{ $this->{Key} };
  $this->{data}->{$key} = \%param;
}

sub exists {
  my ($this, $key) = @_;
  if (! $key) {
    croak "exists() requires a key!\n";
  }
  return exists $this->{data}->{$key};
}

sub read {
  my ($this) = @_;
  local $_;
  my $fd = $this->{FD};
  my $num = scalar @{$this->{Struct}};
  while (<$fd>) {
    chomp;
    my ($key, @field) = split /$this->{Delimiter}/;
    for (my $pos = 0; $pos < $num; $pos++) {
      $this->{data}->{$key}->{ $this->{Struct}->[$pos] } = $this->demask($field[$pos]);
      $this->{data}->{$key}->{$this->{Key}} = $this->demask($key);
    }
  }
}

sub write {
  my ($this) = @_;
  my $fd = $this->{FD};
  seek($fd, 0, 0);
  foreach my $key (keys %{$this->{data}}) {
    my $line = join $this->{Delimiter},
      map { $this->mask($this->{data}->{$key}->{$_}) } @{$this->{Struct}};
    print $fd $key . $this->{Delimiter} . $line . "\n";
  }
}

sub open {
  my ($this) = @_;
  sysopen($this->{FD}, $this->{File}, O_RDWR|O_CREAT, $this->{Mode})
    or croak "Could not open $this->{File}: $!\n";
}

sub close {
  my ($this) = @_;
  $this->{FD}->close();
}

sub lock {
  my ($this) = @_;
  my $fd = $this->{FD};
  if ($this->{Lock}) {
    flock($fd, LOCK_EX) or croak "Could not get LOCK on $this->{File}: $!\n";
  }
}

sub unlock {
  my ($this) = @_;
  $this->close();
}

sub DESTROY {
  my ($this) = @_;
  $this->write();
  $this->unlock();
}

sub mask {
  my ($this, $string) = @_;
  $string =~ s/$this->{Delimiter}/$this->{Mask}/g;
  return $string;
}

sub demask {
  my ($this, $string) = @_;
  $string =~ s/$this->{Mask}/$this->{Delimiter}/g;
  return $string;
}
